class Configuration:
    'This class represents a single configuration, build from traveloptions, for a request'

    @staticmethod
    def build(travelOptions):
        sources = travelOptions.getSources()

        for source in sources:
            source['tm'] = {}
            if 'travelType' in source.keys() and not source['travelType'] is None:
                travelType = source['travelType']
            else:
                travelType = travelOptions.getTravelType().value

            source['tm'][travelType] = {}
            if travelType == 'transit' or travelType == 'biketransit':
                source['tm'][travelType]['frame'] = {}
                source['tm'][travelType]['frame']['time'] = travelOptions.getTravelTime()
                source['tm'][travelType]['frame']['date'] = travelOptions.getTravelDate()
                if not travelOptions.getFrameDuration() is None:
                    source['tm'][travelType]['frame']['duration'] = travelOptions.getFrameDuration()
                if not travelOptions.getRecommendations() is None:
                    source['tm'][travelType]['recommendations'] = travelOptions.getRecommendations()

            if travelType == 'bike':
                # bike specific
                if not travelOptions.getBikeSpeed() is None:
                    source['tm'][travelType]['speed'] = travelOptions.getBikeSpeed()
                if not travelOptions.getBikeUphill() is None:
                    source['tm'][travelType]['uphill'] = travelOptions.getBikeUphill()
                if not travelOptions.getBikeDownhill() is None:
                    source['tm'][travelType]['downhill'] = travelOptions.getBikeDownhill()

            if travelType == 'walk':
                #  walk specific
                if not travelOptions.getWalkSpeed() is None:
                    source['tm'][travelType]['speed'] = travelOptions.getWalkSpeed()
                if not travelOptions.getWalkUphill() is None:
                    source['tm'][travelType]['uphill'] = travelOptions.getWalkUphill()
                if not travelOptions.getWalkDownhill() is None:
                    source['tm'][travelType]['downhill'] = travelOptions.getWalkDownhill()

        cfg = {
            "sources": sources
        }

        if travelOptions.getTravelTimes():
            polygonJson = {
                "values": travelOptions.getTravelTimes(),
                "intersectionMode": travelOptions.getPolygonIntersectionMode().value,
                "serializer": travelOptions.getPolygonSerializationType().value,
                "pointReduction": True,
                "minPolygonHoleSize": travelOptions.getMinPolygonHoleSize()
            }

            if not travelOptions.getBuffer() is None:
                polygonJson['buffer'] = travelOptions.getBuffer()
            if not travelOptions.getSimplifyMeter() is None:
                polygonJson['simplify'] = travelOptions.getSimplifyMeter()
            if not travelOptions.getSrid() is None:
                polygonJson['srid'] = travelOptions.getSrid()
            if not travelOptions.getQuadrantSegments() is None:
                polygonJson['quadrantSegments'] = travelOptions.getQuadrantSegments()

            cfg['polygon'] = polygonJson

        if not travelOptions.getPathSerializer() is None:
            cfg['pathSerializer'] = travelOptions.getPathSerializer().value

        if travelOptions.getTargets():
            cfg['targets'] = travelOptions.getTargets()

        if not travelOptions.getEdgeWeightType() is None:
            cfg['edgeWeightType'] = travelOptions.getEdgeWeightType().value

        if not travelOptions.getMaxEdgeWeight() is None:
            cfg['maxEdgeWeight'] = travelOptions.getMaxEdgeWeight()

        if not travelOptions.getReverse() is None:
            cfg['reverse'] = travelOptions.getReverse()

        if not travelOptions.getDisableCache() is None:
            cfg['disableCache'] = travelOptions.getDisableCache()

        if not travelOptions.getStatisticGroupId() is None:
            cfg['statisticGroupId'] = travelOptions.getStatisticGroupId()

        return cfg



import json
import requests
import logging
import time
from r360_py.util.Configuration import Configuration


class ServiceExecutor:
    'This class should be used to query the service'

    def __init__(self):
        self.execution_time = 0
        return

    def get_execution_time(self):
        return self.execution_time


    def execute_service(self, travel_options, service, object_hook=None, **request_args):

        params = {
            "key": travel_options.getServiceKey(),
            "cfg": json.dumps(Configuration.build(travel_options))
        }

        start_time = time.time()
        response = requests.get(travel_options.getServiceUrl() + 'v1/' + service, params=params, **request_args)
        self.execution_time = time.time() - start_time
        return self.handle_response(service, response, object_hook)


    def execute_service_post(self, travel_options, service, data=None, json=None, object_hook=None, **request_args):

        params = {
            "key": travel_options.getServiceKey()
        }

        start_time = time.time()
        response = requests.post(travel_options.getServiceUrl() + 'v1/' + service, data=data, json=json, params=params, **request_args)
        self.execution_time = time.time() - start_time
        return self.handle_response(service, response, object_hook)


    def handle_response(self, service, response, object_hook=None):
        if response.status_code == 403:
            raise PermissionError(response.text)
        elif response.status_code != 200:
            logging.getLogger("Service").error("status code: " + str(response.status_code) + ", message: " + response.text)
        else:
            if service == "route":
                json_body = response.text.replace("null(", "").rstrip(")")
                return json.loads(json_body, object_hook=object_hook)
            else:
                if (response.text != None and len(response.text) > 0):
                    return json.loads(response.text, object_hook=object_hook)



from r360_py.util.Util import Util
from r360_py.util.enum.EdgeWeightType import EdgeWeightType
from r360_py.util.enum.PathSerializerType import PathSerializerType
from r360_py.util.enum.PolygonIntersectionMode import PolygonIntersectionMode
from r360_py.util.enum.PolygonSerializationType import PolygonSerializationType

class TravelOptions:
    'This class represents the configuration for one request'

    def __init__(self):
        self.sources = []
        self.targets = []
        self.travelTimes = []
        self.travelType = None
        self.buffer = None
        self.simplifyMeter = None
        self.quadrantSegments = None
        self.srid = None
        self.travelDate = Util.currentDate()
        self.travelTime = Util.currentTime()
        self.minPolygonHoleSize = 100000000
        self.intersectionMode = PolygonIntersectionMode.UNION
        self.polygonSerializationType = PolygonSerializationType.JSON
        self.pathSerializer = PathSerializerType.COMPACT_PATH_SERIALIZER
        self.edgeWeightType = EdgeWeightType.TIME
        self.maxEdgeWeight = None
        self.frameDuration = None
        self.reverse = None
        self.bikeSpeed = None
        self.bikeUphill = None
        self.bikeDownhill = None
        self.walkSpeed = None
        self.walkUphill = None
        self.walkDownhill = None
        self.disableCache = None
        self.statisticGroupId = None
        self.recommendations = None

    def setPolygonSerializationType(self, polygonSerializationType):
        self.polygonSerializationType = polygonSerializationType

    def getPolygonSerializationType(self):
        return self.polygonSerializationType

    def setServiceUrl(self, serviceUrl):
        self.serviceUrl = serviceUrl

    def getServiceUrl(self):
        return self.serviceUrl

    def setServiceKey(self, serviceKey):
        self.serviceKey = serviceKey

    def getServiceKey(self):
        return self.serviceKey

    def setSources(self, sources):
        self.sources = sources

    def getSources(self):
        return self.sources

    def addSource(self, source):
        self.sources.append(source)

    def setTargets(self, targets):
        self.targets = targets

    def getTargets(self):
        return self.targets

    def addTarget(self, target):
        self.targets.append(target)

    def setTravelTimes(self, travelTimes):
        self.travelTimes = travelTimes

    def getTravelTimes(self):
        return self.travelTimes

    def addTravelTime(self, travelTime):
        self.travelTimes.append(travelTime)

    def setMinPolygonHoleSize(self, minPolygonHoleSize):
        self.minPolygonHoleSize = minPolygonHoleSize

    def getMinPolygonHoleSize(self):
        return self.minPolygonHoleSize

    def setTravelType(self, travelType):
        self.travelType = travelType

    def getTravelType(self):
        return self.travelType

    def setTravelTime(self, travelTime):
        self.travelTime = travelTime

    def getTravelTime(self):
        return self.travelTime

    def setTravelDate(self, travelDate):
        self.travelDate = travelDate

    def getTravelDate(self):
        return self.travelDate

    def setPolygonIntersectionMode(self, intersectionMode):
        self.intersectionMode = intersectionMode

    def getPolygonIntersectionMode(self):
        return self.intersectionMode

    def setBuffer(self, buffer):
        self.buffer = buffer

    def getBuffer(self):
        return self.buffer

    def setSimplifyMeter(self, simplifyMeter):
        self.simplifyMeter = simplifyMeter

    def getSimplifyMeter(self):
        return self.simplifyMeter

    def setQuadrantSegments(self, quadrantSegments):
        self.quadrantSegments = quadrantSegments

    def getQuadrantSegments(self):
        return self.quadrantSegments

    def setSrid(self, srid):
        self.srid = srid

    def getSrid(self):
        return self.srid

    def setPathSerializer(self, path_serializer):
        self.pathSerializer = path_serializer

    def getPathSerializer(self):
        return self.pathSerializer

    def getEdgeWeightType(self):
        return self.edgeWeightType

    def setEdgeWeightType(self, edge_weight_type):
        self.edgeWeightType = edge_weight_type

    def getMaxEdgeWeight(self):
        return self.maxEdgeWeight

    def setMaxEdgeWeight(self, max_edge_weight):
        self.maxEdgeWeight = max_edge_weight

    def getFrameDuration(self):
        return self.frameDuration

    def setFrameDuration(self, frame_duration):
        self.frameDuration = frame_duration

    def getReverse(self):
        return self.reverse

    def setReverse(self, reverse):
        self.reverse = reverse

    def getBikeSpeed(self):
        return self.bikeSpeed

    def setBikeSpeed(self, bike_speed):
        self.bikeSpeed = bike_speed

    def getBikeUphill(self):
        return self.bikeUphill

    def setBikeUphill(self, bike_uphill):
        self.bikeUphill = bike_uphill

    def getBikeDownhill(self):
        return self.bikeDownhill

    def setBikeDownhill(self, bike_downhill):
        self.bikeDownhill = bike_downhill

    def getWalkSpeed(self):
        return self.walkSpeed

    def setWalkSpeed(self, walk_speed):
        self.walkSpeed = walk_speed

    def getWalkUphill(self):
        return self.walkUphill

    def setWalkUphill(self, walk_uphill):
        self.walkUphill = walk_uphill

    def getWalkDownhill(self):
        return self.walkDownhill

    def setWalkDownhill(self, walk_downhill):
        self.walkDownhill = walk_downhill

    def getDisableCache(self):
        return self.disableCache

    def setDisableCache(self, disable_cache):
        self.disableCache = disable_cache

    def getStatisticGroupId(self):
        return self.statisticGroupId

    def setStatisticGroupId(self, statistic_group_id):
        self.statisticGroupId = statistic_group_id

    def getRecommendations(self):
        return self.recommendations

    def setRecommendations(self, recommendations):
        self.recommendations = recommendations
